@@ -6,9 +6,9 @@ use clap_generate::Shell;
 #[derive(Parser)]
 #[clap(about, version)]
 pub enum App {
-    /// Start playgrounds
-    #[clap(display_order = 1)]
-    Start {
+    /// Initialize playgrounds
+    #[clap(display_order = 4)]
+    Init {
         #[clap(flatten)]
         config: ConfigOpt,
     },
@@ -20,20 +20,6 @@ pub enum App {
         config: ConfigOpt,
     },
 
-    /// Reset playgrounds
-    #[clap(display_order = 3)]
-    Reset {
-        #[clap(flatten)]
-        config: ConfigOpt,
-    },
-
-    /// Start or reset playgrounds
-    #[clap(display_order = 4)]
-    Init {
-        #[clap(flatten)]
-        config: ConfigOpt,
-    },
-
     /// Output shell completion code
     #[clap(display_order = 100)]
     Completion {
