@@ -1,7 +1,7 @@
 use anyhow::{anyhow, bail, Result};
 use async_trait::async_trait;
-use bollard::{container, exec, image, models, Docker};
-use futures::{StreamExt, TryStreamExt};
+use bollard::{container, errors::Error::DockerResponseNotFoundError, exec, image, models, Docker};
+use futures::prelude::*;
 use std::time::Duration;
 
 use crate::store::{PortMap, Store};
@@ -9,35 +9,36 @@ use crate::store::{PortMap, Store};
 #[async_trait]
 pub trait StoreRuntime<T>
 where
-    T: Store + Sync,
+    T: Store + Sync + ?Sized,
 {
     async fn start(&self, store: &T) -> Result<()>;
     async fn create(&self, store: &T) -> Result<models::ContainerCreateResponse>;
     async fn run(&self, store: &T) -> Result<()>;
     async fn stop(&self, store: &T) -> Result<()>;
-    async fn reset(&self, store: &T) -> Result<i64>;
-    async fn ping(&self, store: &T) -> Result<i64>;
+    async fn reset(&self, store: &T) -> Result<()>;
+    async fn ping(&self, store: &T) -> Result<()>;
     async fn wait_ready(&self, store: &T) -> Result<()> {
-        while self.ping(store).await? != 0 {
-            tokio::time::sleep(Duration::SECOND).await
+        while let Err(e) = self.ping(store).await {
+            debug!("docker exec failed: {}", e);
+            tokio::time::sleep(Duration::SECOND).await;
         }
         Ok(())
     }
     async fn init(&self, store: &T) -> Result<()> {
-        match self.reset(store).await {
-            Ok(0) => Ok(()),
-            _ => {
+        self.reset(store)
+            .or_else(async move |e| {
+                debug!("reset failed: {}", e);
                 self.run(store).await?;
                 self.wait_ready(store).await
-            }
-        }
+            })
+            .await
     }
 }
 
 #[async_trait]
 impl<T> StoreRuntime<T> for Docker
 where
-    T: Store + Sync,
+    T: Store + Sync + ?Sized,
 {
     async fn start(&self, store: &T) -> Result<()> {
         Ok(self.start_container::<String>(&store.name(), None).await?)
@@ -82,17 +83,23 @@ where
         self.start(store).await
     }
     async fn stop(&self, store: &T) -> Result<()> {
-        Ok(self.stop_container(&store.name(), None).await?)
+        self.stop_container(&store.name(), None).await.or_else(|e| match e {
+            DockerResponseNotFoundError { .. } => {
+                info!("{} already stopped", store.name());
+                Ok(())
+            }
+            _ => Err(e.into()),
+        })
     }
-    async fn reset(&self, store: &T) -> Result<i64> {
+    async fn reset(&self, store: &T) -> Result<()> {
         exec(self, &store.name(), store.reset_cmd()).await
     }
-    async fn ping(&self, store: &T) -> Result<i64> {
+    async fn ping(&self, store: &T) -> Result<()> {
         exec(self, &store.name(), store.ping_cmd()).await
     }
 }
 
-async fn exec(docker: &Docker, container: &str, cmd: Vec<String>) -> Result<i64> {
+async fn exec(docker: &Docker, container: &str, cmd: Vec<String>) -> Result<()> {
     let id = docker
         .create_exec::<String>(container, exec::CreateExecOptions {
             attach_stdout: Some(true),
@@ -114,6 +121,10 @@ async fn exec(docker: &Docker, container: &str, cmd: Vec<String>) -> Result<i64>
         .await?
         .exit_code
         .ok_or_else(|| anyhow!("exit code is empty"))
+        .and_then(|code| match code {
+            0 => Ok(()),
+            _ => bail!("none-zero exit code: {}", code),
+        })
 }
 
 async fn pull(docker: &Docker, image: &str) -> Result<()> {
